/**
******************************************************************************
* @file    queue_handle.c
* @author  _KEMET, Ltd
* @date    March 2018
* @version Release 1.0.6
* @copyright (c) 2018 _KEMET, Ltd
* @brief   IRPyro evaluation tool with API integration.
*
* @verbatim
 ===============================================================================
                      ##### Description #####
 ===============================================================================
  [..]
  [..]
 @endverbatim
  ******************************************************************************
   @attention <h2><center>&copy; COPYRIGHT 2018 _KEMET, Ltd</center></h2>
   @verbatim
    Copyright (c) 2018, _KEMET, Ltd
    All rights reserved.

    THIS SOFTWARE IS PROVIDED BY _KEMET, Ltd ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL _KEMET, Ltd BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   @endverbatim

  ******************************************************************************
  */
/* Includes ------------------------------------------------------------------*/
#include "queue_handle.h"
/*=============================BEGIN QUEUE HANDLING===========================*/
/* Variables ---------------------------------------------------------*/
static volatile struct Queue Uart_Rx_Buffer= {0,0,{0}};
static volatile struct Queue Uart_Tx_Buffer= {0,0,{0}};
static volatile int TxPrimed = 0;
static volatile int RxOverflow = 0;
/** @defgroup Queue_Handler Queue Handler
  * @ingroup eevt_main
  * @brief Access and handling of circular queues
  * @verbatim
  *
  * @endverbatim
  * @note
  * @{
  */
/** @defgroup queue_handle Queue Handle
  * @brief USART IRQ, Get and Put char into reception and transmission queues
  * @verbatim
  *
  * @endverbatim
  * @note
  * @{
  */
/*------------------------- MESSAGE HANDLE INTERFACE ----------------------------*/
/* manages the USART (PC communications) input buffer as described above */
/**
* @brief interrupt handler for USART 2
* @verbatim
* Uart_Rx_Buffer and Uart_Tx_Buffer are the two queue affected by this function
* On a hardware interrupt a byte will be queued into Uart_Rx_Buffer
* On a software interrupt a byte will be queued into Uart_Tx_Buffer
* @endverbatim
*/
void queue_get_data(void)
{
    uint8_t data;
    // buffer the data (or toss it if there 's no room
    // Flow control will prevent this
    USART_ITConfig(USART2,USART_IT_RXNE,DISABLE) ;
    data = USART_ReceiveData(USART2) & 0xff;
    if (!message_enqueue(&Uart_Rx_Buffer, data))
    {
        RxOverflow = 1;
    }
    USART_ClearFlag(USART2,  USART_FLAG_RXNE);
    USART_ITConfig(USART2,USART_IT_RXNE,ENABLE);
}
void queue_send_data(void)
{
    uint8_t data;
    /* Write one byte to the transmit data register */
    if(message_dequeue(&Uart_Tx_Buffer, &data))
    {
        USART_SendData(USART2 , data);
    }
    else
    {
        // if we have nothing to send , disable the interrupt
        // and wait for a kick
        USART_ITConfig(USART2 , USART_IT_TXE , DISABLE);
        TxPrimed = 0;
    }
}
void queue_handle_enable(void)
{
    USART_callback send_data_to_gui_fn  = queue_send_data;
    USART_callback get_data_from_gui_fn = queue_get_data;
    USART_Callback_Tx_Setup( send_data_to_gui_fn);
    USART_Callback_Rx_Setup( get_data_from_gui_fn);
}
/**
* @brief Gets a character from the receive queue
* @param c pointer to a byte
* @return 0 no data / 1 data
*/
uint8_t message_getchar(uint8_t *c)
{
    uint8_t op_result=0;
    op_result = message_dequeue(&Uart_Rx_Buffer , c);
    return op_result;
}
/**
* @brief sends a character to the transmission queue and enables TX interrupt
* @param c byte to the transmission queue
* @param go_flag signals transmission can begin
*/
void message_putchar(uint8_t c, uint8_t go_flag)
{
    if (go_flag)
    {
        TxPrimed = 1;
        USART_ITConfig(USART2 , USART_IT_TXE , ENABLE);
    }
    else
    {
        message_enqueue(&Uart_Tx_Buffer , c);
    }
}
/** end of queue handle
  * @}
  */
/** end of Queue handler
  * @}
  */
/* ********** Copyright (c) 2018 _KEMET, Ltd. **********END OF FILE************/
